/*****************************************************************************
 * leco                                                                      *
 *****************************************************************************/

/*****************************************************************************
 * File name:     src/block.c                                                *
 * Created:       2006-07-03 by Hampa Hug <hampa@hampa.ch>                   *
 * Last modified: 2006-07-08 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 2006 Hampa Hug <hampa@hampa.ch>                        *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: block.c 48 2006-07-08 11:37:23Z hampa $ */


#include "config.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "lstdint.h"

#include "gf.h"


gf_t par_gf;


void lec_block_clear (uint16_t *blk, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		blk[i] = 0;
	}
}

void lec_block_get (uint16_t *dst, const void *src, unsigned long bsize)
{
	unsigned            i;
	const unsigned char *tmp;

	tmp = src;

	for (i = 0; i < bsize; i++) {
		dst[i] = (tmp[2 * i] << 8) | tmp[2 * i + 1];
	}
}

void lec_block_set (void *dst, const uint16_t *src, unsigned long bsize)
{
	unsigned      i;
	unsigned char *tmp;

	tmp = dst;

	for (i = 0; i < bsize; i++) {
		*(tmp++) = (src[i] >> 8) & 0xff;
		*(tmp++) = src[i] & 0xff;
	}
}

void lec_block_mul (uint16_t *blk, unsigned short x, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		blk[i] = gf_mul (&par_gf, blk[i], x);
	}
}

void lec_block_div (uint16_t *blk, unsigned short x, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		blk[i] = gf_div (&par_gf, blk[i], x);
	}
}

void lec_block_add (uint16_t *dst, const uint16_t *src, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		dst[i] ^= src[i];
	}
}

void lec_block_sub (uint16_t *dst, const uint16_t *src, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		dst[i] ^= src[i];
	}
}

void lec_block_muladd (uint16_t *dst, const uint16_t *src, uint16_t x, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		dst[i] ^= gf_mul (&par_gf, src[i], x);
	}
}

uint16_t lec_get_coeff (unsigned row, unsigned col)
{
	unsigned long x;

	x = (unsigned long) row * (unsigned long) col;

	return (par_gf.exp[x % par_gf.mask]);
}
