/*****************************************************************************
 * leco                                                                      *
 *****************************************************************************/

/*****************************************************************************
 * File name:     src/lecofile.h                                             *
 * Created:       2005-08-11 by Hampa Hug <hampa@hampa.ch>                   *
 * Last modified: 2006-07-08 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 2005-2006 Hampa Hug <hampa@hampa.ch>                   *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: lecofile.h 48 2006-07-08 11:37:23Z hampa $ */


#ifndef LECO_LECOFILE_H
#define LECO_LECOFILE_H 1


#include "config.h"

#include <stdio.h>
#include "lstdint.h"


#define LECO_MAGIC  0x4c45434f
#define LECO_MAGIC1 "[DATA]  "
#define LECO_MAGIC2 "[ECC ]  "
#define LECO_MAGIC3 "[RDNT]  "
#define LECO_HEADER 80


typedef struct {
	FILE               *fp;

	unsigned long      magic;
	unsigned           version;
	unsigned           hsize;
	unsigned long      bsize;
	unsigned long      size;
	unsigned           wsize;
	unsigned long long dofs;
	unsigned long long eofs;
	unsigned long long rofs;
	unsigned           bcnt;
	unsigned           ecnt;
	unsigned long long fsize;
	unsigned char      md5[16];

	unsigned long      fcrc;
	unsigned long      dcrc;
	unsigned long      hcrc;
} lec_file_t;


int lec_seek (FILE *fp, uint64_t pos);
unsigned long lec_read (FILE *fp, uint64_t pos, void *buf, unsigned long cnt);
unsigned long lec_write (FILE *fp, uint64_t pos, const void *buf, unsigned long cnt);

unsigned char *lec_read_buf (FILE *fp, uint64_t pos, unsigned long cnt);

uint16_t lec_get_uint16 (const void *buf, unsigned i);
uint32_t lec_get_uint32 (const void *buf, unsigned i);
uint64_t lec_get_uint64 (const void *buf, unsigned i);

void lec_set_uint16 (void *buf, unsigned i, uint16_t val);
void lec_set_uint32 (void *buf, unsigned i, uint32_t val);
void lec_set_uint64 (void *buf, unsigned i, uint64_t val);

uint32_t lec_get_crc (const void *buf, unsigned long cnt);

void lecf_calc_dofs (lec_file_t *lf);
void lecf_calc_eofs (lec_file_t *lf);
void lecf_calc_rofs (lec_file_t *lf);

int lecf_open (lec_file_t *lf, FILE *fp);

void lecf_init (lec_file_t *lf, FILE *fp);

int lecf_write_header (lec_file_t *lf);
int lecf_write_magic1 (lec_file_t *lf);
int lecf_write_magic2 (lec_file_t *lf);
int lecf_write_magic3 (lec_file_t *lf);
int lecf_write_redundant (lec_file_t *lf, const void *crc);


#endif
